<?php

namespace Drupal\obvious_entity_errors\EventSubscriber;

use Drupal\Core\Entity\EntityDefinitionUpdateManagerInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\Core\Render\RendererInterface;
use Drupal\Core\Session\AccountProxyInterface;
use Drupal\Core\Url;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Symfony\Component\HttpKernel\Event\RequestEvent;
use Symfony\Component\HttpKernel\KernelEvents;

/**
 * Check whether any entity mismatched errors exist and display them.
 */
class ObviousEntityErrorsSubscriber implements EventSubscriberInterface {

  /**
   * The current user.
   *
   * @var \Drupal\Core\Session\AccountProxyInterface
   */
  protected $currentUser;

  /**
   * The messenger service.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected $messenger;

  /**
   * The entity definition update manager.
   *
   * @var \Drupal\Core\Entity\EntityDefinitionUpdateManagerInterface
   */
  protected $entityDefinitionUpdateManager;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The renderer service.
   *
   * @var \Drupal\Core\Render\RendererInterface
   */
  protected $renderer;

  /**
   * Creates an instance of the ObviousEntityErrorsSubscriber class.
   *
   * @param \Drupal\Core\Session\AccountProxyInterface $currentUser
   *   The current user.
   * @param \Drupal\Core\Messenger\MessengerInterface $messenger
   *   The messenger service.
   * @param \Drupal\Core\Entity\EntityDefinitionUpdateManagerInterface $entity_definition_update_manager
   *   The entity definition update manager.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\Core\Render\RendererInterface $renderer
   *   The renderer service.
   */
  public function __construct(
    AccountProxyInterface $currentUser,
    MessengerInterface $messenger,
    EntityDefinitionUpdateManagerInterface $entity_definition_update_manager,
    EntityTypeManagerInterface $entity_type_manager,
    RendererInterface $renderer
  ) {
    $this->currentUser = $currentUser;
    $this->messenger = $messenger;
    $this->entityDefinitionUpdateManager = $entity_definition_update_manager;
    $this->entityTypeManager = $entity_type_manager;
    $this->renderer = $renderer;
  }

  /**
   * Check for entity mismatched errors and display them.
   *
   * @param \Symfony\Component\HttpKernel\Event\RequestEvent $event
   *   The response event, which contains the current request.
   */
  public function checkForEntityErrors(RequestEvent $event) {
    // Only show the entity errors to users with the 'access obvious entity errors' permission.
    if (!$this->currentUser->hasPermission('access obvious entity errors')) {
      return;
    }

    // Check for entity errors.
    if (!$change_list = \Drupal::entityDefinitionUpdateManager()->getChangeSummary()) {
      return;
    }

    $build = [];
    foreach ($change_list as $entity_type_id => $changes) {
      $entity_type = \Drupal::entityTypeManager()->getDefinition($entity_type_id);
      $build[] = [
        '#theme' => 'item_list',
        '#title' => $entity_type->getLabel(),
        '#items' => $changes,
      ];
    }

    $entity_update_issues = \Drupal::service('renderer')->renderInIsolation($build);
    \Drupal::messenger()->addMessage(t('Mismatched entity and/or field definitions.<br/><br/>The following changes were detected in the entity type and field definitions. @updates<br/>For more info on how to resolve the issues, check the <a href="@help-page">help page</a> ', [
      '@updates' => $entity_update_issues,
      '@help-page' => Url::fromUserInput('/admin/help/obvious_entity_errors')->toString(),
    ]), 'error');

  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents(): array {
    $events[KernelEvents::REQUEST][] = ['checkForEntityErrors'];
    return $events;
  }

}
